;-------------------------------------------------------------------------------

Global plane
Global fader
Global scene
Global terrain
Global sky
Global water
Global light

Global entity_type$
Global entity_name$
Global entity_dest$
Global entity_X
Global entity_Y
Global entity_Z
Global entity_angle

Global food_L
Global food_M
Global food_S

Global balloon_L
Global balloon_M
Global balloon_S
Global ring_live
Global ring_dead
Global goal_live
Global goal_dead

Global glow_fore_small
Global glow_fore_large
Global glow_back_small
Global glow_back_large

Global hud_stam_label
Global hud_stam_bar

;-------------------------------------------------------------------------------

Function enter_null()
End Function

Function leave_null()
End Function

Function update_null()
	Return STATE_FADEOUT
End Function

;-------------------------------------------------------------------------------

Function high_write()
	file = WriteFile("data\high.dat")
	
	If file Then
		WriteString(file, high_name(1))
		WriteFloat (file, high_time(1))
		WriteString(file, high_name(2))
		WriteFloat (file, high_time(2))
		WriteString(file, high_name(3))
		WriteFloat (file, high_time(3))
		
		CloseFile(file)
	End If
End Function

Function high_read()
	file = ReadFile("data\high.dat")
	
	If file Then
		high_name(1) = ReadString(file)
		high_time(1) = ReadFloat (file)
		high_name(2) = ReadString(file)
		high_time(2) = ReadFloat (file)
		high_name(3) = ReadString(file)
		high_time(3) = ReadFloat (file)
		
		CloseFile(file)
	End If
End Function

;-------------------------------------------------------------------------------

Global fade_level# = 1

Function fade_out()
	If fade_level < 1.0 Then
		fade_level = fade_level + (1.0 / 30.0)
		value = False
	Else
		fade_level = 1.0
		value = True
	End If

	music_volume(1.0 - fade_level)		
	EntityAlpha fader, fade_level
	Return value
End Function

Function fade_in()
	If fade_level > 0.0 Then
		fade_level = fade_level - (1.0 / 30.0)
		value = False
	Else
		fade_level = 0.0
		value = True
	End If
	
	music_volume(1.0 - fade_level)		
	EntityAlpha fader, fade_level
	Return value
End Function

;-------------------------------------------------------------------------------

Function game_create()

	; Attach a black sprite to the camera for use in "fading" in and out.

	fader = CreateSprite()
	PositionEntity fader, 0, 0, 1.1
	ScaleSprite    fader, 2, 2
	EntityParent   fader, g_mainCamera
	EntityColor    fader, 0, 0, 0
	EntityAlpha    fader, 1

	; Load the player model.

	plane = LoadAnimMesh("data\plane.b3d")
	HideEntity(plane)

	; Load the game entities.

	food_L    = LoadMesh("data\food_L.3ds")
	food_M    = LoadMesh("data\food_M.3ds")
	food_S    = LoadMesh("data\food_S.3ds")
	balloon_L = LoadMesh("data\balloon_L.3ds")
	balloon_M = LoadMesh("data\balloon_M.3ds")
	balloon_S = LoadMesh("data\balloon_S.3ds")
	ring_live = LoadMesh("data\ring_live.3ds")
	ring_dead = LoadMesh("data\ring_dead.3ds")
	goal_live = LoadMesh("data\goal_live.3ds")
	goal_dead = LoadMesh("data\goal_dead.3ds")

	; Make the food a little bigger

	ScaleEntity balloon_L, 2, 2, 2
	ScaleEntity balloon_M, 2, 2, 2
	ScaleEntity balloon_S, 2, 2, 2
	ScaleEntity food_L,    2, 2, 2
	ScaleEntity food_M,    2, 2, 2
	ScaleEntity food_S,    2, 2, 2

	HideEntity food_L
	HideEntity food_M
	HideEntity food_S
	HideEntity balloon_L
	HideEntity balloon_M
	HideEntity balloon_S
	HideEntity ring_live
	HideEntity ring_dead
	HideEntity goal_live
	HideEntity goal_dead
	
	; Load the glowing ring halos.
	
	glow_fore_large = LoadSprite("data\glow.png", 2)
	glow_fore_small = LoadSprite("data\glow.png", 2)
	glow_back_large = LoadSprite("data\glow.png", 2)
	glow_back_small = LoadSprite("data\glow.png", 2)
	
	large = 2.0 * course_radius
	small = 1.9 * course_radius
	
	ScaleEntity    glow_fore_large, large, large, large
	ScaleEntity    glow_fore_small, small, small, small
	ScaleEntity    glow_back_large, large, large, large
	ScaleEntity    glow_back_small, small, small, small
	SpriteViewMode glow_fore_large, 2
	SpriteViewMode glow_fore_small, 2
	SpriteViewMode glow_back_large, 2
	SpriteViewMode glow_back_small, 2
	RotateEntity   glow_back_large, 0, 180, 0
	RotateEntity   glow_back_small, 0, 180, 0
	HideEntity     glow_fore_large
	HideEntity     glow_fore_small
	HideEntity     glow_back_large
	HideEntity     glow_back_small
	
	; Load the environment models.
	
	scene = LoadBSP("data\scene.bsp")
	BSPAmbientLight scene, 127, 127, 127
	PositionEntity  scene, 512, 0, 512
	ScaleEntity     scene, 0.125, 0.125, 0.125
	EntityType      scene, COLLISION_LEVEL

	; Load the terrain

	terrain = LoadTerrain("data\height.bmp")
	shadow  = LoadTexture("data\shadow.png", 16 + 32 + 8)
	texture = LoadTexture("data\color.png")
	size    = TerrainSize(terrain)

	ScaleTexture  texture, size, size
	ScaleTexture  shadow, 16, 16	
	TerrainDetail terrain, 2000, True
	ScaleEntity   terrain, 1, max_altitude, 1
	EntityTexture terrain, texture
	EntityTexture terrain, shadow, 0, 1
	EntityType    terrain, COLLISION_LEVEL
	EntityFX      terrain, 1
	FreeTexture   texture
	
	; Create the sky sphere
	
	sky     = CreateSphere(32)
	texture = LoadTexture("data\sky.png")
	EntityTexture sky, texture
	ScaleEntity   sky, -max_distance, max_distance, max_distance
	EntityFX      sky, 1 + 8
	FreeTexture   texture
	
	; Create the water plane	
	
	water   = CreatePlane()
	texture = LoadTexture("data\water.png")
	PositionEntity water, 0, -1, 0
	EntityTexture  water, texture
	EntityFX       water, 1
	FreeTexture    texture

	; Create a light source

	light = CreateLight(2)
	PositionEntity light, size / 2, max_altitude, size / 2
	
	; Set up the camera properties

	camera = SL_GetRightCamera()
	CameraFogMode  camera, 1
	CameraFogColor camera, 255, 255, 255
	CameraFogRange camera, max_distance * 0.5, max_distance * 0.9
	CameraRange    camera,                1.0, max_distance + 1.0

	camera = SL_GetLeftCamera()
	CameraFogMode  camera, 1
	CameraFogColor camera, 255, 255, 255
	CameraFogRange camera, max_distance * 0.5, max_distance * 0.9
	CameraRange    camera,                1.0, max_distance + 1.0

	; Temporarily position the camera far from the origin.

	PositionEntity g_mainCamera, 1000, 1000, 1000
	
	; Configure collisions
	
	Collisions COLLISION_PARTICLE, COLLISION_LEVEL, 2, 1
	Collisions COLLISION_PLAYER,   COLLISION_LEVEL,  2, 2
	Collisions COLLISION_PLAYER,   COLLISION_COURSE, 1, 2
	Collisions COLLISION_PLAYER,   COLLISION_HEALTH, 1, 1
	
	TextureFilter "chrome", 64
	
	; Load HUD images.

	hud_stam_label = LoadSprite("data\stamina_label.png", 2 + 16 + 32)
	hud_stam_bar   = LoadSprite("data\stamina_bar.png",   2 + 16 + 32)

	ScaleSprite hud_stam_label, 2.0, 0.25
	ScaleSprite hud_stam_bar,   2.0, 0.25
	HideEntity  hud_stam_label
	HideEntity  hud_stam_bar
	EntityOrder hud_stam_label, -2
	EntityOrder hud_stam_bar,   -1

	; timeColon must be created before timeNum(ic) are set.

	colon     = LoadTexture("data\colon.png", 2)
	number(0) = LoadTexture("data\num0.png", 2)
	number(1) = LoadTexture("data\num1.png", 2)
	number(2) = LoadTexture("data\num2.png", 2)
	number(3) = LoadTexture("data\num3.png", 2)
	number(4) = LoadTexture("data\num4.png", 2)
	number(5) = LoadTexture("data\num5.png", 2)
	number(6) = LoadTexture("data\num6.png", 2)
	number(7) = LoadTexture("data\num7.png", 2)
	number(8) = LoadTexture("data\num8.png", 2)
	number(9) = LoadTexture("data\num9.png", 2)
	
	; radar images
	radarBG =    LoadTexture("data\radar_background.png", 2)
	radarPoint = LoadTexture("data\radar_point.png", 2)
	
	sound_init()
	
	; Set some default high scores, in case the high score file is missing.
	
	high_name(1) = "LOL"
	high_name(2) = "OMG"
	high_name(3) = "WTF"
	high_name(4) = "BBQ"
	high_time(1) = 180.00
	high_time(2) = 180.00
	high_time(3) = 180.00
	high_time(4) =   0.00
	
	high_read()
	
End Function

Function game_delete()
	sound_free()
	
	FreeEntity sky
	FreeEntity water
	FreeEntity terrain
	FreeEntity plane
	FreeEntity fader
	FreeEntity scene
	FreeEntity light
	
	FreeEntity food_L
	FreeEntity food_M
	FreeEntity food_S
	FreeEntity balloon_L
	FreeEntity balloon_M
	FreeEntity balloon_S
	FreeEntity ring_live
	FreeEntity ring_dead
	FreeEntity goal_live
	FreeEntity goal_dead
	
	FreeEntity glow_fore_large
	FreeEntity glow_fore_small
	FreeEntity glow_back_large
	FreeEntity glow_back_small
	
	FreeTexture shadow
	
	FreeEntity hud_stam_bar
	FreeEntity hud_stam_label
End Function

;-------------------------------------------------------------------------------

; Run the game update for the current state.

Function game_update()
	Select curr_state
		Case STATE_NULL:    next_state = update_null()
		Case STATE_TITLE:   next_state = update_title()
		Case STATE_SELECT:  next_state = update_select()
		Case STATE_START:   next_state = update_start()
		Case STATE_LEVEL:   next_state = update_level()
		Case STATE_CRASH:   next_state = update_crash()
		Case STATE_FADEOUT: next_state = update_fadeout()
		Case STATE_GOAL:    next_state = update_goal()
		Case STATE_HIGH:    next_state = update_high()
	End Select
End Function

; Handle any state transition triggered by the previous update.

Function game_state()
	If next_state <> curr_state Then
		Select curr_state
			Case STATE_NULL:    leave_null()
			Case STATE_TITLE:   leave_title()
			Case STATE_SELECT:  leave_select()
			Case STATE_START:   leave_start()
			Case STATE_LEVEL:   leave_level()
			Case STATE_CRASH:   leave_crash()
			Case STATE_FADEOUT: leave_fadeout()
			Case STATE_GOAL:    leave_goal()
			Case STATE_HIGH:    leave_high()
		End Select
	
		prev_state = curr_state
		curr_state = next_state	
		
		Select curr_state
			Case STATE_NULL:   enter_null()
			Case STATE_TITLE:  enter_title()
			Case STATE_SELECT: enter_select()
			Case STATE_START:  enter_start()
			Case STATE_LEVEL:  enter_level()
			Case STATE_CRASH:  enter_crash()
			Case STATE_GOAL:   enter_goal()
			Case STATE_HIGH:   enter_high()
		End Select
	End If	
End Function

;-------------------------------------------------------------------------------