;-------------------------------------------------------------------------------

Function player_create(X#, Y#, Z#, A#)
	P.Player     = New Player

	P\camera      = g_mainCamera
	P\camera_mode = CAMERA_FOLLOW
	P\entity      = CopyEntity(plane)
	P\health      = 1.0
	P\starttime   = 0

	P\X  = X
	P\Y  = Y
	P\Z  = Z
	P\VX = 0.0
	P\VY = 0.0
	P\VZ = 0.0

	; Position the player at the start of the course
	
	ShowEntity     P\entity
	EntityRadius   P\entity, player_radius
	EntityType     P\entity, COLLISION_PLAYER
	PositionEntity P\entity, X, Y, Z
	RotateEntity   P\entity, 0, A-180, 0

	; Set up the propeller sprite
	
	P\propeller = LoadSprite("data\prop.png", 2, P\entity)
	SpriteViewMode P\propeller, 2
	ScaleEntity    P\propeller, 0.5, 0.5, 0.5
	
	; Show the plane's shadow.
	
	TextureBlend shadow, 2
	
	; Create HUD elements.

	EntityParent   hud_stam_label, P\camera
	EntityParent   hud_stam_bar,   P\camera
	PositionEntity hud_stam_label, 0.8, 1.9, 3
	PositionEntity hud_stam_bar,   0.8, 1.9, 3

	P\timer = create_timer()
	PositionEntity P\timer, -2.0, 1.9, 3
	HideEntity     P\timer
	
	; create radar.
	P\radar = radar_create()
	HideEntity P\radar
	; create north marker for radar.
	P\radarNorth = LoadSprite("data\radar_north.png", 2, P\radar)
	ScaleSprite P\radarNorth, 0.1, 0.1
	; need to make sure the north marker is "always on top"
	EntityOrder P\radarNorth, -2
	; place "north" marker on the radar.
	PositionEntity P\radarNorth, 0, 0.66, 0
	
	; create plane marker
	P\radarPlane = LoadSprite("data\plane.png", 2, P\radar)
	ScaleSprite P\radarPlane, 0.15, 0.15
	EntityOrder P\radarPlane, -3
	
End Function

Function player_delete()
	P.Player = First Player
	FreeEntity P\propeller
	FreeEntity P\entity
	FreeEntity P\timer
	
	HideEntity hud_stam_label
	HideEntity hud_stam_bar
	
	FreeEntity P\radar
	Delete P
End Function

;-------------------------------------------------------------------------------

Function player_crash()
	P.Player = First Player

	; Load the pieces of broken plane.

	part1 = LoadMesh("data\body.b3d")
	part2 = LoadMesh("data\tail.b3d")
	part3 = LoadMesh("data\bike.b3d")
	part4 = LoadMesh("data\left_wing.b3d")
	part5 = LoadMesh("data\right_wing.b3d")
	part6 = LoadMesh("data\dude.b3d")

	; Parent pieces to the player, so they appear in the same place as the plane.

	EntityParent part1, P\entity, 0
	EntityParent part2, P\entity, 0
	EntityParent part3, P\entity, 0
	EntityParent part4, P\entity, 0
	EntityParent part5, P\entity, 0
	EntityParent part6, P\entity, 0

	; Compute scatter velocity based on player velocity and impact normal.

	VX# = (P\VX + 2.0 * CollisionNX(P\entity, 1)) * 5.0
	VY# = (P\VY + 2.0 * CollisionNY(P\entity, 1)) * 5.0
	VZ# = (P\VZ + 2.0 * CollisionNZ(P\entity, 1)) * 5.0
	
	particle_create(part1, VX, VY, VZ)
	particle_create(part2, VX, VY, VZ)
	particle_create(part3, VX, VY, VZ)
	particle_create(part4, VX, VY, VZ)
	particle_create(part5, VX, VY, VZ)
	particle_create(part6, VX, VY, VZ)

	; Hide the original plane entity, so the pieces appear to replace it.

	Animate      P\entity, 0
	EntityAlpha  P\entity, 0
	EntityAlpha  P\propeller, 0
	
	; Make the shadow disappear.
	
	TextureBlend shadow, 0
	
	; Play the appropriate sound.
		
	ChannelVolume channel_squeak, 0.0

	If EntityY(P\entity) < 10.0 And CollisionNY(P\entity, 1) > 0.9 Then
		PlaySound(sound_splash)
	Else
		PlaySound(sound_crash)
	End If
End Function

;-------------------------------------------------------------------------------

Function input_update()
	P.Player = First Player

	throttle# = P\throttle

	; Check for an input mode change.

	If KeyDown(59) Then P\control_mode = CONTROL_JOYSTICK
	If KeyDown(60) Then P\control_mode = CONTROL_KEYBOARD

	If P\control_mode = CONTROL_JOYSTICK Then
	
		; Handle joystick input.

		If P\health > 0 Then
			If JoyDown(1) Then
				P\throttle = 1.0
				ChannelVolume channel_squeak, 1.0
			Else
				P\throttle = default_throttle
				ChannelVolume channel_squeak, 0.0
			End If
		Else
			P\throttle = 0.0
		End If
		
		P\roll_rate  = JoyX() * -2.00
		P\pitch_rate = JoyY() * -2.00
		
	Else
	
		; Handle keyboard input.
		
		If P\health > 0 Then
			If KeyDown(57) Then
				P\throttle = 1.0
				ChannelVolume channel_squeak, 1.0
			Else
				P\throttle = default_throttle
				ChannelVolume channel_squeak, 0.0
			End If
		Else
			P\throttle = 0.0
		End If
		
		If KeyDown(205) Or KeyDown(203) Then		
			If KeyDown(205) Then P\roll_rate  = -2.00
			If KeyDown(203) Then P\roll_rate  = +2.00
		Else
			P\roll_rate = 0.0
		End If
		
		If KeyDown(200) Or KeyDown(208) Then
			If KeyDown(200) Then P\pitch_rate =  2.00
			If KeyDown(208) Then P\pitch_rate = -2.00
		Else
			P\pitch_rate = 0.0
		End If
	End If

	; Prevent the player from flying upside-down.  (It's easier that way.)

	If EntityRoll(P\entity) >=  90.0 And P\roll_rate > 0.0 Then P\roll_rate = 0.0
	If EntityRoll(P\entity) <= -90.0 And P\roll_rate < 0.0 Then P\roll_rate = 0.0

	; If the throttle changed, reset the animation speed.
	
	If P\health > 0 Then
		If throttle <> P\throttle Then
			Animate P\entity, 1, (P\throttle - default_throttle) * 3
			P\propeller_rate = P\throttle * 10
		End If
	Else
		Animate P\entity, 0
	End If
End Function

;-------------------------------------------------------------------------------

Function camera_update()
	P.Player = First Player
	
	; Compute the ideal camera position from the player position.

	P\dest_viewX# = EntityX(P\entity, True)
	P\dest_viewY# = EntityY(P\entity, True)
	P\dest_viewZ# = EntityZ(P\entity, True)
	
	If P\camera_mode = CAMERA_FOLLOW Then
		TFormVector 0, 0, -1, P\entity, 0	
		P\dest_viewX# = P\dest_viewX# + TFormedX() * camera_dz
		P\dest_viewY# = P\dest_viewY# + TFormedY() * camera_dz
		P\dest_viewZ# = P\dest_viewZ# + TFormedZ() * camera_dz
	Else
		TFormVector 1, 0,  0, P\entity, 0	
		P\dest_viewX# = P\dest_viewX# + TFormedX() * camera_dz
		P\dest_viewY# = P\dest_viewY# + TFormedY() * camera_dz
		P\dest_viewZ# = P\dest_viewZ# + TFormedZ() * camera_dz
	End If
					
	TFormVector 0, 1,  0, P\entity, 0	
	P\dest_viewX# = P\dest_viewX# + TFormedX() * camera_dy
	P\dest_viewY# = P\dest_viewY# + TFormedY() * camera_dy
	P\dest_viewZ# = P\dest_viewZ# + TFormedZ() * camera_dy

	; Interpolate the current position toward the ideal position.
	
	N# = 8
	P\curr_viewX# = (P\curr_viewX# * (N-1) + P\dest_viewX#) / N
	P\curr_viewY# = (P\curr_viewY# * (N-1) + P\dest_viewY#) / N
	P\curr_viewZ# = (P\curr_viewZ# * (N-1) + P\dest_viewZ#) / N
	
	; Move and point the camera
	
	PositionEntity P\camera, P\curr_viewX#, P\curr_viewY#, P\curr_viewZ#
	PointEntity    P\camera, P\entity

	; Recenter the sky sphere about the camera.
	
	PositionEntity sky, P\curr_viewX#, P\curr_viewY#, P\curr_viewZ#
	
End Function

Function shadow_update()
	P.Player = First Player

	; Move the shadow.

	U# = (       EntityX(P\entity, True)) / 16
	V# = (size - EntityZ(P\entity, True)) / 16

	; Blitz applies texture translation before rotation.  This complicates things.
		
	Y# = EntityYaw(P\entity, True)
	D# = Sqr(U * U + V * V)
	A# = ATan2(V, U) + Y
	
	PositionTexture shadow, (D * Cos(A) - 0.5), (D * Sin(A) - 0.5)
	RotateTexture   shadow, Y
End Function

;-------------------------------------------------------------------------------

Function flight_update()
	P.Player = First Player

	TFormVector 0, 0, 1, P\entity, 0
	FX# = TFormedX()
	FY# = TFormedY()
	FZ# = TFormedZ()

	TFormVector 0, 1, 0, P\entity, 0
	UX# = TFormedX()
	UY# = TFormedY()
	UZ# = TFormedZ()

	; Gravity acts toward the ground

	VX# =  0.0
	VY# = -9.0 / 30.0
	VZ# =  0.0

	; Thrust acts forward, as a function of health.
	
	K# = (1.0 - P\health)
	H# = 1 - K * K * K
	
	VX = VX + FX * H * P\throttle * thrust_coefficient
	VY = VY + FY * H * P\throttle * thrust_coefficient
	VZ = VZ + FZ * H * P\throttle * thrust_coefficient

	k# = Sqr(P\VX * P\VX + P\VY * P\VY + P\VZ * P\VZ)

	; Lift acts upward, porportionally to velocity
		
	VX = VX + UX * k * lift_coefficient
	VY = VY + UY * k * lift_coefficient
	VZ = VZ + UZ * k * lift_coefficient
	
	N# = 16
	P\VX = (P\VX * (N-1) + VX) / N
	P\VY = (P\VY * (N-1) + VY) / N
	P\VZ = (P\VZ * (N-1) + VZ) / N

	AlignToVector P\entity, P\VX, P\VY, P\VZ, 3, 0.1

	; Prevent the user from climbing or descending too quickly.

	If EntityPitch(P\entity) >  45.0 Then
		RotateEntity P\entity,  45.0, EntityYaw(P\entity), EntityRoll(P\entity)
	End If
	If EntityPitch(P\entity) < -45.0 Then
		RotateEntity P\entity, -45.0, EntityYaw(P\entity), EntityRoll(P\entity)
	End If

	X# = EntityX(P\entity) + P\VX
	Y# = EntityY(P\entity) + P\VY
	Z# = EntityZ(P\entity) + P\VZ

	PositionEntity P\entity, X, Y, Z
	
	; Update the dynamic sounds
	
	ocean_volume((64.0 - Y) / 64.0)
	wind_volume(k)	

	If P\health > 0 Then
		ChannelPitch channel_squeak, 22050 + (H - 0.5) * P\throttle * 10000
	Else
		ChannelVolume channel_squeak, 0.0
	End If
End Function

Function player_update()
	P.Player = First Player

	input_update()
	flight_update()

	; Deduct health while pedaling.

	If Not freeflight Then
		If P\health > 0 Then
			P\health = P\health - (P\throttle - default_throttle) * health_coefficient / 30.0
		Else
			P\health = 0.0
		End If
	End If
		
	; Spin the propeller
	
	TurnEntity P\propeller, 0, 0, P\propeller_rate

	; Check if the player has hit the next checkpoint or completed the course.

	If First Checkpoint = Null Then
		time# = P\lasttime - P\starttime
		
		ChannelVolume channel_squeak, 0.0

		If (Not freeflight) And (time < high_time(selection))
			PlaySound(sound_ooh)
			Return STATE_HIGH
		Else
			PlaySound(sound_wow)
			Return STATE_GOAL
		End If
	Else
		C.Checkpoint = First Checkpoint

		If EntityCollided(P\entity, COLLISION_COURSE) = C\entity Then
			PlaySound(sound_checkpoint)
			course_ring()
		End If	
	End If
	
	; Check if the player has grabbed food.
	
	If EntityCollided(P\entity, COLLISION_HEALTH) Then
		PlaySound(sound_eat)
		course_food()
	End If
	
	; Check if the player crashed.
	
	If EntityCollided(P\entity, COLLISION_LEVEL) Then
		player_crash()
		Return STATE_CRASH
	End If
	
	TurnEntity P\entity, P\pitch_rate, 0, P\roll_rate
	
	; update hud.
	hud_update()
	
	Return STATE_LEVEL
End Function

;-------------------------------------------------------------------------------

Function player_debug()
	P.Player = First Player
	X# = EntityX(P\entity)
	Y# = EntityY(P\entity)
	Z# = EntityZ(P\entity)
	DebugLog X + " " + Y + " " + Z
End Function

;-------------------------------------------------------------------------------

Function hud_update()
	P.Player = First Player
	
	; time on the hud.
	; first off, if the time is 0, we are *just* beginning.

	If(P\starttime = 0) Then
		P\starttime = Float(MilliSecs()) / 1000.0
		P\lasttime = -1
		ShowEntity P\timer
		ShowEntity hud_stam_bar
		ShowEntity hud_stam_label
		ShowEntity P\radar
	EndIf
	
	; stamina on the hud.
	
	ScaleSprite hud_stam_bar, 2.0 * P\health, 0.25
;	ScaleSprite hud_stam_bar, 0.5 * P\health, 0.12
	
	; display time digits.

	time# = Float(MilliSecs()) / 1000.0

	set_timer(P\timer, time - P\starttime)
	P\lasttime = time

	
	; radar on the hud.
	radar_update()
	
End Function

;-------------------------------------------------------------------------------

; Radar love! (bad reference to b-52's, i think) [one-hit-wonder Golden Earring]

;------------

Function radar_create()
	radar = CreateSprite(g_mainCamera)
	; hidden right away because well, we get some funky stuff if we don't.
	HideEntity radar
	
	ScaleSprite radar, 0.75, 0.75
	EntityTexture radar, radarBG
	PositionEntity radar, 2.15, -1.55, 3
	EntityOrder radar, -1
	
	For C.Checkpoint = Each Checkpoint
		FreeEntity C\radarPoint
		C\radarPoint = CreateSprite(radar)
		ScaleSprite C\radarPoint, 0.1, 0.1
		EntityTexture C\radarPoint, radarPoint
		PositionEntity C\radarPoint, 0, 0, 0
		; force it to draw on top of the radar, which is order -1.
		EntityOrder C\radarPoint, -2
	Next
	
	Return radar
End Function

;-------------------------------------------------------------------------------

Function radar_update()
	P.Player = First Player
	
	; really need a way to go *backwards* in the list, but we can suffer for now.
	For C.Checkpoint = Each Checkpoint
		; position checkpoint.. we won't worry about the overhead of positioning the
		; 'first' checkpoint, but will accept it as overhead for now.
		set_radar_position(C, EntityX(P\entity), EntityY(P\entity), EntityZ(P\entity))
	Next
	
	; set color, etc, to first checkpoint.
	C.Checkpoint = First Checkpoint
	If(Not C = Null) Then
		If(EntityY(P\entity) < C\Y - 2.0) Then
			EntityColor C\radarPoint, (C\Y - EntityY(P\entity)) * 20, 255, 0
		Else If(EntityY(P\entity) > C\Y + 2.0) Then
			EntityColor C\radarPoint, 0, 255 - (EntityY(P\entity) - C\Y) * 20, (EntityY(P\entity) - C\Y) * 20
		Else
			EntityColor C\radarPoint, 0, 255, 0
		EndIf
		set_radar_position(C, EntityX(P\entity), EntityY(P\entity), EntityZ(P\entity))
	EndIf
	
	; need to rotate radar so that the top of the radar corresponds to the
	; plane's direction.
	; comment out this line to to radar tweaking (when lining up the points
	; on the radar with the radar itself).
	RotateEntity P\radar, 0, 0, -EntityYaw(P\entity)
End Function

;-------------------------------------------------------------------------------

Function radar_delete()
	;P.Player = First Player
	;FreeEntity P\radar
	;FreeEntity radarpoint
End Function

;-------------------------------------------------------------------------------

Function set_radar_position(ckpoint.CheckPoint, playerx#, playery#, playerz#)
	threshhold# = 0.56
	distx# = (ckpoint\x - playerx) / 200.0
	;disty# = (ckpoint\Y - playery) / 200.0
	distz# = (ckpoint\z - playerz) / 200.0
	distance# = Sqr(distx * distx + distz *distz)
	If distance > threshhold Then
		distx# = distx / distance * threshhold
		distz# = distz / distance * threshhold
	EndIf
	;DebugLog "chkpoint: " + ckpoint\x + "," + ckpoint\y + "," + ckpoint\z
	;DebugLog "player:   " + playerx   + "," + playery
	;DebugLog "Dist: " + distx + "," + disty + "," + distz
	;PositionEntity ckpoint\radarPoint, distx-0.23, distz+0.15, -0.3
	PositionEntity ckpoint\radarPoint, distx, distz, 0
End Function